@@ -20,6 +20,7 @@ import json
 import logging
 import threading
 import traceback
+import typing
 from collections import OrderedDict
 from contextlib import contextmanager
 from os import environ
@@ -41,6 +42,7 @@ from opentelemetry import trace as trace_api
 from opentelemetry.attributes import BoundedAttributes
 from opentelemetry.sdk import util
 from opentelemetry.sdk.environment_variables import (
+    OTEL_ATTRIBUTE_COUNT_LIMIT,
     OTEL_ATTRIBUTE_VALUE_LENGTH_LIMIT,
     OTEL_EVENT_ATTRIBUTE_COUNT_LIMIT,
     OTEL_LINK_ATTRIBUTE_COUNT_LIMIT,
@@ -61,11 +63,12 @@ from opentelemetry.util._time import _time_ns
 
 logger = logging.getLogger(__name__)
 
+_DEFAULT_OTEL_ATTRIBUTE_COUNT_LIMIT = 128
 _DEFAULT_OTEL_SPAN_ATTRIBUTE_COUNT_LIMIT = 128
-_DEFAULT_OTEL_SPAN_EVENT_COUNT_LIMIT = 128
-_DEFAULT_OTEL_SPAN_LINK_COUNT_LIMIT = 128
 _DEFAULT_OTEL_EVENT_ATTRIBUTE_COUNT_LIMIT = 128
 _DEFAULT_OTEL_LINK_ATTRIBUTE_COUNT_LIMIT = 128
+_DEFAULT_OTEL_SPAN_EVENT_COUNT_LIMIT = 128
+_DEFAULT_OTEL_SPAN_LINK_COUNT_LIMIT = 128
 
 
 _ENV_VALUE_UNSET = ""
@@ -219,7 +222,7 @@ class ConcurrentMultiSpanProcessor(SpanProcessor):
         self,
         func: Callable[[SpanProcessor], Callable[..., None]],
         *args: Any,
-        **kwargs: Any
+        **kwargs: Any,
     ):
         futures = []
         for sp in self._span_processors:
@@ -442,12 +445,10 @@ class ReadableSpan:
         if self.parent is not None:
             if isinstance(self.parent, Span):
                 ctx = self.parent.context
-                parent_id = "0x{}".format(
-                    trace_api.format_span_id(ctx.span_id)
-                )
+                parent_id = f"0x{trace_api.format_span_id(ctx.span_id)}"
             elif isinstance(self.parent, SpanContext):
-                parent_id = "0x{}".format(
-                    trace_api.format_span_id(self.parent.span_id)
+                parent_id = (
+                    f"0x{trace_api.format_span_id(self.parent.span_id)}"
                 )
 
         start_time = None
@@ -484,12 +485,8 @@ class ReadableSpan:
     @staticmethod
     def _format_context(context):
         x_ctx = OrderedDict()
-        x_ctx["trace_id"] = "0x{}".format(
-            trace_api.format_trace_id(context.trace_id)
-        )
-        x_ctx["span_id"] = "0x{}".format(
-            trace_api.format_span_id(context.span_id)
-        )
+        x_ctx["trace_id"] = f"0x{trace_api.format_trace_id(context.trace_id)}"
+        x_ctx["span_id"] = f"0x{trace_api.format_span_id(context.span_id)}"
         x_ctx["trace_state"] = repr(context.trace_state)
         return x_ctx
 
@@ -539,19 +536,23 @@ class SpanLimits:
     Limit precedence:
 
     - If a model specific limit is set, it will be used.
+    - Else if the corresponding global limit is set, it will be used.
     - Else if the model specific limit has a default value, the default value will be used.
-    - Else if model specific limit has a corresponding global limit, the global limit will be used.
+    - Else if the global limit has a default value, the default value will be used.
 
     Args:
-        max_attributes: Maximum number of attributes that can be added to a Span.
-            Environment variable: OTEL_SPAN_ATTRIBUTE_COUNT_LIMIT
-            Default: {_DEFAULT_SPAN_ATTRIBUTE_COUNT_LIMIT}
+        max_attributes: Maximum number of attributes that can be added to a span, event, and link.
+            Environment variable: OTEL_ATTRIBUTE_COUNT_LIMIT
+            Default: {_DEFAULT_ATTRIBUTE_COUNT_LIMIT}
         max_events: Maximum number of events that can be added to a Span.
             Environment variable: OTEL_SPAN_EVENT_COUNT_LIMIT
             Default: {_DEFAULT_SPAN_EVENT_COUNT_LIMIT}
         max_links: Maximum number of links that can be added to a Span.
             Environment variable: OTEL_SPAN_LINK_COUNT_LIMIT
             Default: {_DEFAULT_SPAN_LINK_COUNT_LIMIT}
+        max_span_attributes: Maximum number of attributes that can be added to a Span.
+            Environment variable: OTEL_SPAN_ATTRIBUTE_COUNT_LIMIT
+            Default: {_DEFAULT_OTEL_SPAN_ATTRIBUTE_COUNT_LIMIT}
         max_event_attributes: Maximum number of attributes that can be added to an Event.
             Default: {_DEFAULT_OTEL_EVENT_ATTRIBUTE_COUNT_LIMIT}
         max_link_attributes: Maximum number of attributes that can be added to a Link.
@@ -569,16 +570,14 @@ class SpanLimits:
         max_attributes: Optional[int] = None,
         max_events: Optional[int] = None,
         max_links: Optional[int] = None,
+        max_span_attributes: Optional[int] = None,
         max_event_attributes: Optional[int] = None,
         max_link_attributes: Optional[int] = None,
         max_attribute_length: Optional[int] = None,
         max_span_attribute_length: Optional[int] = None,
     ):
-        self.max_attributes = self._from_env_if_absent(
-            max_attributes,
-            OTEL_SPAN_ATTRIBUTE_COUNT_LIMIT,
-            _DEFAULT_OTEL_SPAN_ATTRIBUTE_COUNT_LIMIT,
-        )
+
+        # span events and links count
         self.max_events = self._from_env_if_absent(
             max_events,
             OTEL_SPAN_EVENT_COUNT_LIMIT,
@@ -589,17 +588,32 @@ class SpanLimits:
             OTEL_SPAN_LINK_COUNT_LIMIT,
             _DEFAULT_OTEL_SPAN_LINK_COUNT_LIMIT,
         )
+
+        # attribute count
+        global_max_attributes = self._from_env_if_absent(
+            max_attributes, OTEL_ATTRIBUTE_COUNT_LIMIT
+        )
+        self.max_attributes = (
+            global_max_attributes or _DEFAULT_OTEL_ATTRIBUTE_COUNT_LIMIT
+        )
+
+        self.max_span_attributes = self._from_env_if_absent(
+            max_span_attributes,
+            OTEL_SPAN_ATTRIBUTE_COUNT_LIMIT,
+            global_max_attributes or _DEFAULT_OTEL_SPAN_ATTRIBUTE_COUNT_LIMIT,
+        )
         self.max_event_attributes = self._from_env_if_absent(
             max_event_attributes,
             OTEL_EVENT_ATTRIBUTE_COUNT_LIMIT,
-            _DEFAULT_OTEL_EVENT_ATTRIBUTE_COUNT_LIMIT,
+            global_max_attributes or _DEFAULT_OTEL_EVENT_ATTRIBUTE_COUNT_LIMIT,
         )
         self.max_link_attributes = self._from_env_if_absent(
             max_link_attributes,
             OTEL_LINK_ATTRIBUTE_COUNT_LIMIT,
-            _DEFAULT_OTEL_LINK_ATTRIBUTE_COUNT_LIMIT,
+            global_max_attributes or _DEFAULT_OTEL_LINK_ATTRIBUTE_COUNT_LIMIT,
         )
 
+        # attribute length
         self.max_attribute_length = self._from_env_if_absent(
             max_attribute_length,
             OTEL_ATTRIBUTE_VALUE_LENGTH_LIMIT,
@@ -612,16 +626,7 @@ class SpanLimits:
         )
 
     def __repr__(self):
-        return "{}(max_span_attributes={}, max_events_attributes={}, max_link_attributes={}, max_attributes={}, max_events={}, max_links={}, max_attribute_length={})".format(
-            type(self).__name__,
-            self.max_span_attribute_length,
-            self.max_event_attributes,
-            self.max_link_attributes,
-            self.max_attributes,
-            self.max_events,
-            self.max_links,
-            self.max_attribute_length,
-        )
+        return f"{type(self).__name__}(max_span_attributes={self.max_span_attributes}, max_events_attributes={self.max_event_attributes}, max_link_attributes={self.max_link_attributes}, max_attributes={self.max_attributes}, max_events={self.max_events}, max_links={self.max_links}, max_attribute_length={self.max_attribute_length})"
 
     @classmethod
     def _from_env_if_absent(
@@ -656,13 +661,14 @@ _UnsetLimits = SpanLimits(
     max_attributes=SpanLimits.UNSET,
     max_events=SpanLimits.UNSET,
     max_links=SpanLimits.UNSET,
+    max_span_attributes=SpanLimits.UNSET,
     max_event_attributes=SpanLimits.UNSET,
     max_link_attributes=SpanLimits.UNSET,
     max_attribute_length=SpanLimits.UNSET,
     max_span_attribute_length=SpanLimits.UNSET,
 )
 
-# not remove for backward compat. please use SpanLimits instead.
+# not removed for backward compat. please use SpanLimits instead.
 SPAN_ATTRIBUTE_COUNT_LIMIT = SpanLimits._from_env_if_absent(
     None,
     OTEL_SPAN_ATTRIBUTE_COUNT_LIMIT,
@@ -732,7 +738,7 @@ class Span(trace_api.Span, ReadableSpan):
         self._limits = limits
         self._lock = threading.Lock()
         self._attributes = BoundedAttributes(
-            self._limits.max_attributes,
+            self._limits.max_span_attributes,
             attributes,
             immutable=False,
             max_value_len=self._limits.max_span_attribute_length,
@@ -759,9 +765,7 @@ class Span(trace_api.Span, ReadableSpan):
             self._links = BoundedList.from_seq(self._limits.max_links, links)
 
     def __repr__(self):
-        return '{}(name="{}", context={})'.format(
-            type(self).__name__, self._name, self._context
-        )
+        return f'{type(self).__name__}(name="{self._name}", context={self._context})'
 
     def _new_events(self):
         return BoundedList(self._limits.max_events)
@@ -889,9 +893,7 @@ class Span(trace_api.Span, ReadableSpan):
                 self.set_status(
                     Status(
                         status_code=StatusCode.ERROR,
-                        description="{}: {}".format(
-                            exc_type.__name__, exc_val
-                        ),
+                        description=f"{exc_type.__name__}: {exc_val}",
                     )
                 )
 
@@ -1088,11 +1090,6 @@ class TracerProvider(trace_api.TracerProvider):
         self._span_limits = span_limits or SpanLimits()
         self._atexit_handler = None
 
-        self._resource._attributes = BoundedAttributes(
-            self._span_limits.max_attributes,
-            self._resource._attributes,
-            max_value_len=self._span_limits.max_attribute_length,
-        )
         if shutdown_on_exit:
             self._atexit_handler = atexit.register(self.shutdown)
 
@@ -1103,18 +1100,23 @@ class TracerProvider(trace_api.TracerProvider):
     def get_tracer(
         self,
         instrumenting_module_name: str,
-        instrumenting_library_version: str = "",
+        instrumenting_library_version: typing.Optional[str] = None,
+        schema_url: typing.Optional[str] = None,
     ) -> "trace_api.Tracer":
         if not instrumenting_module_name:  # Reject empty strings too.
             instrumenting_module_name = ""
             logger.error("get_tracer called with missing module name.")
+        if instrumenting_library_version is None:
+            instrumenting_library_version = ""
         return Tracer(
             self.sampler,
             self.resource,
             self._active_span_processor,
             self.id_generator,
             InstrumentationInfo(
-                instrumenting_module_name, instrumenting_library_version
+                instrumenting_module_name,
+                instrumenting_library_version,
+                schema_url,
             ),
             self._span_limits,
         )
